@@ -1,3 +1,10 @@
+from base64 import encodestring
+from datetime import datetime
+import logging
+import mimetypes
+import re
+import urllib
+
 from django.db import models
 from django.db.models import Q
 from django.contrib.auth.models import User
@@ -7,14 +14,13 @@ from django.core.cache import cache
 from django.conf import settings
 from django.template.defaultfilters import slugify, striptags
 from django.utils.translation import ugettext_lazy as _
-from datetime import datetime
-from base64 import encodestring
-import mimetypes
-import re
-import urllib
+
+from decorators import logtime, once_per_instance
 
 WORD_LIMIT = getattr(settings, 'ARTICLES_TEASER_LIMIT', 75)
 AUTO_TAG = getattr(settings, 'ARTICLES_AUTO_TAG', True)
+DEFAULT_DB = getattr(settings, 'ARTICLES_DEFAULT_DB', 'default')
+LOOKUP_LINK_TITLE = getattr(settings, 'ARTICLES_LOOKUP_LINK_TITLE', True)
 
 MARKUP_HTML = 'h'
 MARKUP_MARKDOWN = 'm'
@@ -34,21 +40,32 @@ DEFAULT_ADDTHIS_USER = getattr(settings, 'DEFAULT_ADDTHIS_USER', None)
 
 # regex used to find links in an article
 LINK_RE = re.compile('<a.*?href="(.*?)".*?>(.*?)</a>', re.I|re.M)
-TITLE_RE = re.compile('<title>(.*?)</title>', re.I|re.M)
+TITLE_RE = re.compile('<title.*?>(.*?)</title>', re.I|re.M)
 TAG_RE = re.compile('[^a-z0-9\-_\+\:\.]?', re.I)
 
+log = logging.getLogger('articles.models')
+
 def get_name(user):
     """
     Provides a way to fall back to a user's username if their full name has not
     been entered.
     """
+
     key = 'username_for_%s' % user.id
+
+    log.debug('Looking for "%s" in cache (%s)' % (key, user))
     name = cache.get(key)
     if not name:
+        log.debug('Name not found')
+
         if len(user.get_full_name().strip()):
+            log.debug('Using full name')
             name = user.get_full_name()
         else:
+            log.debug('Using username')
             name = user.username
+
+        log.debug('Caching %s as "%s" for a while' % (key, name))
         cache.set(key, name, 86400)
 
     return name
@@ -56,6 +73,7 @@ User.get_name = get_name
 
 class Tag(models.Model):
     name = models.CharField(max_length=64, unique=True)
+    slug = models.CharField(max_length=64, unique=True, null=True, blank=True)
 
     def __unicode__(self):
         return self.name
@@ -64,28 +82,39 @@ class Tag(models.Model):
     def clean_tag(name):
         """Replace spaces with dashes, in case someone adds such a tag manually"""
 
-        name = name.replace(' ', '-')
+        name = name.replace(' ', '-').encode('ascii', 'ignore')
         name = TAG_RE.sub('', name)
-        return name.lower().strip()
+        clean = name.lower().strip()
+
+        log.debug('Cleaned tag "%s" to "%s"' % (name, clean))
+        return clean
 
     def save(self, *args, **kwargs):
         """Cleans up any characters I don't want in a URL"""
 
-        self.name = Tag.clean_tag(self.name)
+        log.debug('Ensuring that tag "%s" has a slug' % (self,))
+        self.slug = Tag.clean_tag(self.name)
         super(Tag, self).save(*args, **kwargs)
 
     @models.permalink
     def get_absolute_url(self):
-        return ('articles_display_tag', (self.name,))
+        return ('articles_display_tag', (self.cleaned,))
+
+    @property
+    def cleaned(self):
+        """Returns the clean version of the tag"""
+
+        return self.slug or Tag.clean_tag(self.name)
 
     @property
     def rss_name(self):
-        return u'tags/%s' % self.name
+        return self.cleaned
 
     class Meta:
         ordering = ('name',)
 
 class ArticleStatusManager(models.Manager):
+
     def default(self):
         default = self.all()[:1]
 
@@ -112,10 +141,11 @@ class ArticleStatus(models.Model):
             return self.name
 
 class ArticleManager(models.Manager):
+
     def active(self):
         """
-        Retrieves all active articles which have been published and have not yet
-        expired.
+        Retrieves all active articles which have been published and have not
+        yet expired.
         """
         now = datetime.now()
         return self.get_query_set().filter(
@@ -175,9 +205,7 @@ class Article(models.Model):
     objects = ArticleManager()
 
     def __init__(self, *args, **kwargs):
-        """
-        Make sure that we have some rendered content to use.
-        """
+        """Makes sure that we have some rendered content to use"""
 
         super(Article, self).__init__(*args, **kwargs)
 
@@ -198,10 +226,9 @@ class Article(models.Model):
         return self.title
 
     def save(self, *args, **kwargs):
-        """
-        Renders the article using the appropriate markup language.
-        """
-        using = kwargs.get('using', 'default')
+        """Renders the article using the appropriate markup language."""
+
+        using = kwargs.get('using', DEFAULT_DB)
 
         self.do_render_markup()
         self.do_addthis_button()
@@ -245,7 +272,7 @@ class Article(models.Model):
 
         return False
 
-    def do_unique_slug(self, using):
+    def do_unique_slug(self, using=DEFAULT_DB):
         """
         Ensures that the slug is always unique for the year this article was
         posted
@@ -287,28 +314,40 @@ class Article(models.Model):
 
         return False
 
-    def do_auto_tag(self, using):
+    @logtime
+    @once_per_instance
+    def do_auto_tag(self, using=DEFAULT_DB):
         """
         Performs the auto-tagging work if necessary.
 
         Returns True if an additional save is required, False otherwise.
         """
 
+        if not self.auto_tag:
+            log.debug('Article "%s" (ID: %s) is not marked for auto-tagging. Skipping.' % (self.title, self.pk))
+            return False
+
+        # don't clobber any existing tags!
+        existing_ids = [t.id for t in self.tags.all()]
+        log.debug('Article %s already has these tags: %s' % (self.pk, existing_ids))
+
+        unused = Tag.objects.all()
+        if hasattr(unused, 'using'):
+            unused = unused.using(using)
+        unused = unused.exclude(id__in=existing_ids)
+
         found = False
-        if self.auto_tag:
-            # don't clobber any existing tags!
-            existing_ids = [t.id for t in self.tags.all()]
-            unused = Tag.objects.using(using).exclude(id__in=existing_ids)
-            for tag in unused:
-                regex = re.compile(r'\b%s\b' % tag.name, re.I)
-                if regex.search(self.content) or regex.search(self.title) or \
-                   regex.search(self.description) or regex.search(self.keywords):
-                    self.tags.add(tag)
-                    found = True
+        to_search = (self.content, self.title, self.description, self.keywords)
+        for tag in unused:
+            regex = re.compile(r'\b%s\b' % tag.name, re.I)
+            if any(regex.search(text) for text in to_search):
+                log.debug('Applying Tag "%s" (%s) to Article %s' % (tag, tag.pk, self.pk))
+                self.tags.add(tag)
+                found = True
 
         return found
 
-    def do_default_site(self, using):
+    def do_default_site(self, using=DEFAULT_DB):
         """
         If no site was selected, selects the site used to create the article
         as the default site.
@@ -317,12 +356,15 @@ class Article(models.Model):
         """
 
         if not len(self.sites.all()):
-            self.sites.add(Site.objects.using(using).get(pk=settings.SITE_ID))
+            sites = Site.objects.all()
+            if hasattr(sites, 'using'):
+                sites = sites.using(using)
+            self.sites.add(sites.get(pk=settings.SITE_ID))
             return True
 
         return False
 
-    def get_unique_slug(self, slug, using):
+    def get_unique_slug(self, slug, using=DEFAULT_DB):
         """Iterates until a unique slug is found"""
 
         # we need a publish date before we can do anything meaningful
@@ -334,7 +376,11 @@ class Article(models.Model):
         counter = 1
 
         while True:
-            not_unique = Article.objects.using(using).filter(publish_date__year=year, slug=slug)
+            not_unique = Article.objects.all()
+            if hasattr(not_unique, 'using'):
+                not_unique = not_unique.using(using)
+            not_unique = not_unique.filter(publish_date__year=year, slug=slug)
+
             if len(not_unique) == 0:
                 return slug
 
@@ -350,49 +396,48 @@ class Article(models.Model):
         used as the title.  Once a title is determined, it is cached for a week
         before it will be requested again.
         """
-        links = {}
-        keys = []
+
+        links = []
 
         # find all links in the article
+        log.debug('Locating links in article: %s' % (self,))
         for link in LINK_RE.finditer(self.rendered_content):
             url = link.group(1)
+            log.debug('Do we have a title for "%s"?' % (url,))
             key = 'href_title_' + encodestring(url).strip()
 
             # look in the cache for the link target's title
-            if not cache.get(key):
-                try:
-                    # open the URL
-                    c = urllib.urlopen(url)
-                    html = c.read()
-                    c.close()
+            title = cache.get(key)
+            if title is None:
+                log.debug('Nope... Getting it and caching it.')
+                title = link.group(2)
 
-                    # try to determine the title of the target
-                    title = TITLE_RE.search(html)
-                    if title: title = title.group(1)
-                    else: title = link.group(2)
-                except:
-                    # if anything goes wrong (ie IOError), use the link's text
-                    title = link.group(2)
+                if LOOKUP_LINK_TITLE:
+                    try:
+                        log.debug('Looking up title for URL: %s' % (url,))
+                        # open the URL
+                        c = urllib.urlopen(url)
+                        html = c.read()
+                        c.close()
+
+                        # try to determine the title of the target
+                        title_m = TITLE_RE.search(html)
+                        if title_m:
+                            title = title_m.group(1)
+                            log.debug('Found title: %s' % (title,))
+                    except:
+                        # if anything goes wrong (ie IOError), use the link's text
+                        log.warn('Failed to retrieve the title for "%s"; using link text "%s"' % (url, title))
 
                 # cache the page title for a week
+                log.debug('Using "%s" as title for "%s"' % (title, url))
                 cache.set(key, title, 604800)
 
-            # get the link target's title from cache
-            val = cache.get(key)
-            if val:
-                # add it to the list of links and titles
-                links[url] = val
+            # add it to the list of links and titles
+            if url not in (l[0] for l in links):
+                links.append((url, title))
 
-                # don't duplicate links to the same page
-                if url not in keys: keys.append(url)
-
-        # now go thru and sort the links according to where they appear in the
-        # article
-        sorted = []
-        for key in keys:
-            sorted.append((key, links[key]))
-
-        return tuple(sorted)
+        return tuple(links)
     links = property(_get_article_links)
 
     def _get_word_count(self):
@@ -451,6 +496,7 @@ class Article(models.Model):
 
     class Meta:
         ordering = ('-publish_date', 'title')
+        get_latest_by = 'publish_date'
 
 class Attachment(models.Model):
     upload_to = lambda inst, fn: 'attach/%s/%s/%s' % (datetime.now().year, inst.article.slug, fn)
